if(!dojo._hasResource["com.ibm.mm.catalog.opensearch.ResultParser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.catalog.opensearch.ResultParser"] = true;
dojo.provide("com.ibm.mm.catalog.opensearch.ResultParser");

dojo.require("com.ibm.mm.catalog.opensearch.ResultModel");
//dojo.require("com.ibm.mashups.enabler.xml.XPath");
dojo.require("dojox.data.dom");

/**
 * A parser that can be used to parse Mashup Hub's open search result.
 * The parsed result is an instance of <tt>ResultModel</tt>.
 */
dojo.declare("com.ibm.mm.catalog.opensearch.MashupHubResultParser", null, {
    constructor: function(){
        //namespaces that used in XPath expressions
		var ns = {
            "atom": "http://www.w3.org/2005/Atom",
            "catalog": "http://www.ibm.com/xmlns/atom/opensearch/feeds/1.0/",
			"os": "http://a9.com/-/spec/opensearch/1.1/"
        };
		//XPath to get all the entries' content
        var path = "/atom:feed/atom:entry";
		//XPath to get all the links of the ATOM feed document
        var linkPath = "/atom:feed/atom:link";
		//XPath to get the total result of the ATOM feed document
		var totalResultPath = "/atom:feed/os:totalResults";
		
        
		/**
		 * Return first child of <tt>parent</tt> of type <tt>nodeType</tt>
		 * @param {Object} parent the DOM node
		 * @param {Number} nodeType the type of the DOM nodes to consider, refer to W3C DOM spec.
		 * @return {Object} the DOM node
		 */
        function getFirstChildOfType(/* DOM node */parent, /* DOM node type */ nodeType){
            var childrenNodes = parent.childNodes;
            for (var i = 0, n = childrenNodes.length; i < n; i++) {
                if (childrenNodes.item(i).nodeType == nodeType) {
                    return childrenNodes.item(i);
                }
            }
        }
        
		/**
		 * Parse the <tt>content</tt> element of the feed's entry
		 * @param {Object} content A JSON object contains all the sub-elements of the <tt>content</tt> element
		 */
        function parseEntryContent(/* DOM node */ content, contentObj){
			
           	var catalogNode = getFirstChildOfType(content, 1);
            var childrenNodes = catalogNode.childNodes;
            
            for (var i = 0, n = childrenNodes.length; i < n; i++) {
                var node = childrenNodes.item(i);
                if (node.nodeType === 1) {
                    var nodeValue;
					if ("catalog:l10n" == node.nodeName){
						// add parsing for l18n section introduced recently for ECM space template feed update
						var titleObj = {}, descObj = {};
						var titles = node.getElementsByTagName("catalog:title")[0];
						// WEBKIT HOTFIX 
						// TODO: use com.ibm.mashups.enabler.xml.XPath.evaluateEntry 
						// to fetch elements instead
						if (!titles) {
							titles = node.getElementsByTagName("title")[0];
						}
						var titleNodes = titles.childNodes;
						for (k=0;k<titleNodes.length;k++){
							var oneTitle = titleNodes[k];
							if (oneTitle.nodeType === 1){
								// #17824 the title in feed could be empty for some lang, check it like check desc
								if(oneTitle.childNodes.length > 0) {
									titleObj[oneTitle.getAttributeNode("lang").nodeValue] = oneTitle.childNodes[0].nodeValue;
								} else {
									titleObj[oneTitle.getAttributeNode("lang").nodeValue] = "";
								}
							}
						}
						
						var descs = node.getElementsByTagName("catalog:description")[0]; 
						// WEBKIT HOTFIX 
						// TODO: use com.ibm.mashups.enabler.xml.XPath.evaluateEntry 
						// to fetch elements instead
						if (!descs) {
							descs = node.getElementsByTagName("description")[0];
						}
						var descNodes = descs.childNodes;
						for (j=0;j<descNodes.length;j++){
							var oneDesc = descNodes[j];
							if (oneDesc.nodeType === 1){
								if (oneDesc.childNodes.length > 0){
									descObj[oneDesc.getAttributeNode("lang").nodeValue] = oneDesc.childNodes[0].nodeValue;
								} else {
									descObj[oneDesc.getAttributeNode("lang").nodeValue] = "";
								}
							}
						}
						contentObj["title"] = titleObj;
						contentObj["description"] = descObj;
						// end
					} else{
	                    if ("catalog:tags" != node.nodeName) {
	                        nodeValue = dojo.trim(dojox.data.dom.textContent(node));
	                    } else {
							//Flatten the tag elements into an array                 					
	                        var tagsNode = node.getElementsByTagName("catalog:tag");
	                        if(tagsNode.length === 0){
	                            tagsNode = node.getElementsByTagName("tag"); 
	                        }	
	                        var tags = [];
	                        dojo.forEach(tagsNode, function(tagNode){
	                            tags.push(getFirstChildOfType(tagNode, 3).nodeValue);			
	                        });										//JSLINT-IGNORE: no problem here
	                        nodeValue = tags;
	                   }
					   var nodeName = node.nodeName;
					   if (nodeName.indexOf("catalog:") === 0){
					   		 nodeName = nodeName.substring(8);
					   }
	                   contentObj[nodeName] = nodeValue;						
					}
                }
            }
			
			//special treatment of the element "dateModified", need to multiply by 1000, so it can be consumed by 
			//a JavaScript Date object.
			if (contentObj.dateModified) {
			  contentObj.dateModified = parseInt(contentObj.dateModified) * 1000;		//JSLINT-IGNORE
			}
			
            return contentObj;
        }
		
        function parseEntry(/* DOM node */ entry) {
			var entryObj = {};
			for (var i=0, n=entry.childNodes.length; i<n; i++) {
				var node = entry.childNodes[i];
				if ("content" == node.nodeName) {
					parseEntryContent(node, entryObj);
				} else if ("category" == node.nodeName) {
					entryObj.category = node.getAttribute("term");
				} 
			}
			return entryObj;
		}
		/**
		 * Parse the <tt>link</tt> elements of the feed
		 * @param {Object} link DOM node
		 * @return {Object} A JSON object contains all the attributes of the <tt>link</tt> element
		 */
        function parseLink(/* DOM node */ link){
            var attributes = link.attributes;
            var linkObj = {};
            for (var i = 0, n = attributes.length; i < n; i++) {
                var attr = attributes.item(i);
                linkObj[attr.name] = attr.value;
            }
            return linkObj;
        }
		/**
		 * parse the <tt>totalResults</tt> element of the feed
		 * @param (Object) xmlDoc An XML document
		 * @return (Number) A number which represents the total result of the searching
		 */
        function parseTotalResult(/* XML DOM Object */xmlDoc){
			var totalResultNodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(totalResultPath, xmlDoc, ns);
			if (totalResultNodes.length) {
				var totalResultNode = getFirstChildOfType(totalResultNodes[0], 3);
				var totalResult = dojo.trim(dojox.data.dom.textContent(totalResultNode));
				return totalResult;
			}
		}
		
		/**
		 * Parse the XML document to get the <tt>ResultModel</tt>.
		 * @param {Object} xmlDoc An XML document
		 * @return {Object} ResultModel
		 */
        this.parse = function(/* XML DOM Object */ xmlDoc){
			var entriesObj = [];
            var linksObj = [];
            var nodes = [];
			var linkNodes = [];

			try {
				nodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(path, xmlDoc, ns);
            	linkNodes = com.ibm.mashups.enabler.xml.XPath.evaluateXPath(linkPath, xmlDoc, ns);
			} catch (e) {
				throw e; // Re-throw the exception
			}
 			
			//parse the content of the entry
            dojo.forEach(nodes, function(node){
                entriesObj.push(parseEntry(node));
            });
            
			//parse the links of the feed
            dojo.forEach(linkNodes, function(node){
                linksObj.push(parseLink(node));
            });
            
			//determine whether there exists more result items
            var hasNext = false;
            for (var i = 0, n = linksObj.length; i < n; i++) {
                if (linksObj[i].rel == "next") {
                    hasNext = true;
                    break;
                }
            }
            var totalResult = parseTotalResult(xmlDoc);
			
            return new com.ibm.mm.catalog.opensearch.ResultModel(entriesObj, hasNext, totalResult);
        };
    }
});

//Create a manager that manages all the parsers
(function(){
    if (typeof com.ibm.mm.catalog.opensearch.ResultParserManager == "undefined") {
        com.ibm.mm.catalog.opensearch.ResultParserManager = {
            "mashuphub": new com.ibm.mm.catalog.opensearch.MashupHubResultParser()
        };
    }
})();

}
