/* Copyright IBM Corp. 2010  All Rights Reserved. */
/* 
 * @author Ben
 * Summary: Generic navigation iWidget that is controlled by a NavigationController
 * 			using a selection model and tree model.
 */
dojo.provide("com.ibm.mm.builder.systemwidget.ControlledNavWidget");
dojo.require("com.ibm.pb.iwidget.iWidgetBase");
dojo.require("com.ibm.widgets.NavigationWidget");
dojo.require("com.ibm.dnd.PageCreator");

dojo.declare("com.ibm.mm.builder.systemwidget.ControlledNavWidget", com.ibm.pb.iwidget.iWidgetBase, {
	
	controller: null,
	
	dndOperable: false,
	
	varyLevelCondition: false,
	
	//TODO: Mashup 3.0 temp fixes for Mashup navigation problem, all code related to varyLevelInSpace is required.
	varyLevelInSpace: false,
	
	attachNode: null,
	
	firstItemClass: null,
	
	editorIWidgetID: null,
	
	selectionModel: null,
	
	isSwitching: false,
	
	onLoad: function(){
		var mname="onLoad", lgr = this.LOGGER, me = this;
		lgr.entering(mname, arguments);
		
		var attrs = this.iContext.getiWidgetAttributes();
		
		var attrMix = this.getItemSetValues([
			"dndType","dndAccept","firstItemClass","attachNode","editorIWidgetID","varyLevelCondition", // Strings
			"openDelay","popupDelay","closeDelay","level","levels", // Numbers
			"baseClass","tabListClass","tabClass","class","selectedClass",	// Strings
			"showInlineEditor","tiedToSelection","dndOperable","varyLevelInSpace",// Boolean
			"item","controller","dndCreator"	// Object
			]);
		this.convertMapTypes(attrMix, ["openDelay","popupDelay","closeDelay","level","levels"], "Number");
		this.convertMapTypes(attrMix, ["showInlineEditor","tiedToSelection","dndOperable"], "Boolean");
		this.convertMapTypes(attrMix, ["item","controller","dndCreator"], "Object");
		
		if(attrMix.editorIWidgetID) {
			this.editorIWidgetID = attrMix.editorIWidgetID;
		}
		
		dojo.forEach(["dndOperable","controller","firstItemClass","attachNode","varyLevelCondition","varyLevelInSpace"], function(key){
			this[key] = attrMix[key];
			delete attrMix[key];
		}, this);
		
		this.baseLevel = attrMix.level;
		if ((this.varyLevelInSpace && this.controller.getCurrentSpaceId()) ||
		     this.controller.getCurrentSpaceId() == "FAKE_SPACE" ) {  //20070 if we have a fake space and don't increment attrMix.level then
		     														  // the nav label for the page doesn't display		
			// Increment the level by one if we're in a space
			attrMix.level++;
		}
		
		this.selectionModel = this.controller.getSelectionModel();
		
		var initArgs = dojo.delegate({
			model: this.controller.getTreeModel(),
			selectionModel: this.selectionModel,
			tiedToSelection: true,
			skipPreload: true,
			openDelay: 0,
			closeDelay: 500,
			popupDelay: 500,
			dndClass: "com.ibm.pb.dnd.ControlledModeledSource",
			dndType: "nm",
			dndAccept: "nm",
			dndEnabled: false,
			dndCreator: com.ibm.dnd.PAGE_CREATOR.creator,
			showInlineEditor: this.showInlineEditor,
			baseClass: "",
			tabListClass: "lotusInlinelist lotusLinks",
			tabClass: "",
			"class": "",
			selectedClass: "lotusSelected",
//			getVariedLevel: function() {
//				if(me.checkVaryLevel()) {
//					return me.baseLevel + 1;
//				}
//				else {
//					return me.baseLevel;
//				}
//			},
			inlineEditorWidgetParams: {
				onClick: function() {
					var cb = dojo.hitch(me, function(){
						com.ibm.pb.iwidget.control.onLoad(me.editorIWidgetID, function(){
							me.evtSvc.broadcastEvent(me.builderEvents.CREATE_PAGE, {
							    spaceId: me.controller.getCurrentSpaceId(),
							    parentPageId: me.widget.model.getIdentity(me.widget.item)
							});
						});
					});
					if(com.ibm.pb.contextMenu.sharedActions.widgetEditModeChecker()) {
						// skip the page dirty check and display another confirm dialog if there is unsaved widget
						return;
					}
					if(com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().isDirty()){
						var cws = com.ibm.mm.builder.coreWidgetsStrings;
						var saveConfirmDlg = new com.ibm.mm.builder.dialog.PageSaveConfirmDialog({
					        contentWarning: cws.I_PAGE_SAVE_WARNING,
					        contentQuestion: cws.I_PAGE_SAVE_SUGGEST,
					        btnSaveText: cws.I_COMMON_SAVE,
					        btnNotSaveText: cws.I_PAGE_SAVE_NO_SAVE,
					        btnCancelText: cws.I_PAGE_SAVE_KEEP_EDIT,
					        callbackSave: cb,
					        callbackNotSave: cb,
							hideNotSave: false,
							callbackCancel: function(){}
					    });
					    saveConfirmDlg.show();
					} else {
						cb();
					}
				}
			},
			onModelItemClick: function(widget, item){
				lgr.trace("closure", "SWITCH_PAGE called via onModelItemClick ${0} ${1}", [widget, item]);
				me.evtSvc.broadcastEvent(me.builderEvents.SWITCH_PAGE, this.model.getIdentity(item));
			}
		}, attrMix);
		
		if(this.attachNode) {
			this.attachNode = dojo.byId(this.attachNode);
		}
		if(!this.attachNode) {
			this.attachNode = this.iContext.getRootElement();
		}
		
		this.widget = new com.ibm.widgets.NavigationWidget(initArgs, this.attachNode);
		
		if(this.firstItemClass) {
			dojo.connect(this.widget, "onLoad", this.widget, function(){
				var ch = this.getChildren();
				if(ch.length > 0) {
					dojo.addClass(ch[0].domNode, me.firstItemClass);
				}
			});
		}
		
		this.widget.startup();
		com.ibm.mm.builder.utils.navigationUtil.updateSpaceTitleInNavBar();
		lgr.exiting(mname, arguments);
	},
	
//	checkVaryLevel: function() {
//		var mname="checkVaryLevel", lgr = this.LOGGER;
//		lgr.entering(mname, arguments);
//		var ret = false;
//		if(this.varyLevelCondition && (!this.widget || this.widget.level == this.baseLevel)) {
//			lgr.trace(mname, "Checking varyLevelCondition");
//			ret = eval(this.varyLevelCondition);
//		}
//		lgr.exiting(mname, [ret]);
//		return ret;
//	},
	onPageSwitched:function(iEvent){
		var mname="onPageSwitched", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		if(this.dndOperable) {
			this.widget.disableDnd();
		}
		if(this.varyLevelInSpace && this.controller.getCurrentSpaceId() && this.widget.level == this.baseLevel) {
			lgr.trace(mname, "Navigated into a space, reset level of nav widget ${0} to ${1}", [this, this.baseLevel + 1]);
			this.widget.attr("level", this.baseLevel + 1);
			this.refresh(iEvent.payload);
		}
		if(this.varyLevelInSpace && !this.controller.getCurrentSpaceId() && this.widget.level == this.baseLevel + 1) {
			lgr.trace(mname, "Navigated into a space, reset level of nav widget ${1} to ${0}", [this, this.baseLevel]);
			this.widget.attr("level", this.baseLevel);
			this.refresh(iEvent.payload);
		}
		com.ibm.mm.builder.utils.navigationUtil.updateSpaceTitleInNavBar();
		lgr.exiting(mname, arguments);
	},
	onPageModeChanged: function(iEvent){
		// summary: toggle drag and drop, view mode disabled, edit mode active
		var mname="onPageModeChanged", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		if(this.dndOperable) {
			if(iEvent.payload === "edit") {
				this.widget.setupDnd();
			}
			else {
				this.widget.disableDnd();
			}
		}
		// show/hide the 'new page' link according to the user previlidge
		if(iEvent.payload === "edit") {
			var pid = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
			var show = com.ibm.pb.contextMenu.sharedActions.confirmCreateSiblingPage(pid);
			this._showHideInlineEditor(show);
		}
		lgr.exiting(mname, arguments);
	},
	onRefreshNavigation:function(iEvent){
		var mname="onRefreshNavigation", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		try{
			this.refresh(iEvent.payload);
		}catch(e){
			var msg = dojo.string.substitute(com.ibm.mm.builder.coreWidgetsMessages.E_NAV_REFRESH_NAVIGATION_1,[e.message]);
			lgr.severe(mname,msg);
		}
		lgr.exiting(mname, arguments);
	},
	refresh: function(pid){	
		var mname="refresh", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		this.widget.reload(true);
		lgr.exiting(mname, arguments);
	},
	
	onRefreshSpace: function(iEvent){
		var mname="onRefreshSpace", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		com.ibm.mm.builder.utils.navigationUtil.updateSpaceTitleInNavBar();
		lgr.exiting(mname, arguments);
	},
	
	onUnload: function(){
		var mname="onUnload", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
	    if (this.widget) {
			this.widget.destroy(true);
	        this.widget = null;
	    }
	    lgr.exiting(mname, arguments);
	},
	
	onOpenPage: function(iEvent) {
		this.controller.onOpenPage(iEvent);
	},
	
	onClosePage: function(iEvent){
		this.controller.onClosePage(iEvent);
	},
	
	_showHideInlineEditor: function(show) {
		if (this.widget.inlineEditor) {
			var node = this.widget.inlineEditor.domNode;
			if (show) {
				node.style.display = "";
			}
			else {
				node.style.display = "none";
			}
		}
	}
});