/* Copyright IBM Corp. 2013  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.data.model");

/**
   @ibm-module Legacy
   @author <a href="mailto:litong01@us.ibm.com">Tong Li</a>      
   @class com.ibm.mashups.data 
   @name com.ibm.mashups.data 
*/ 
dojo.declare("com.ibm.mashups.data.model", null, 
/** @lends com.ibm.mashups.data.model */
{
	/**
	* Return the value of a given data item for the requested attribute. <p/>
	* Saying that an "item x does not have a value for an attribute y"
    * is identical to saying that an "item x does not have attribute y". 
    * If data.hasAttribute(item, attribute) returns false, then
    * data.getValue(item, attribute) will return undefined.
	
 	* @param {Object} item the data item returned to the callback when the fetch is complete. 
 	* @param {String} attribute the name of a data attribute.
 	* @type Object
 	*/		
	getValue: function( /* Object */ item, /* String */ attribute) {
	}, 
    
	/**
	* Return the type of the value of a given data item for the requested attribute. <p/>
    * If data.hasAttribute(item, attribute) returns false, then
    * data.getValue(item, attribute) will return undefined.
	
 	* @param {Object} item the data item returned to the callback when the fetch is complete. 
 	* @param {String} attribute the name of a data attribute.
 	* @returns {String} return the type of the value
 	*/		
	getValueType: function( /* Object */ item, /* String */ attribute) {
	}, 
    
    /** summary:
    * This getValues() method works just like the getValue() method, but getValues()
    * always returns an array rather than a single attribute value.  The array
    * may be empty, may contain a single attribute value, or may contain many
    * attribute values.
    * If the item does not have a value for the given attribute, then getValues()
    * will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
    * returns false, then store.getValues(item, attribute) will return [].)
    * It should always return an array even though it could be empty. 
    *
    * @param {item} item The item to access values on.
    * @param {string} attribute The attribute to access represented as a string.
    * @type Array
 	*/
	getValues: function(/* Object */ item,
                        /* String */ attribute) {
    },
      
	/**      
	* summary:
    * Returns an array with all the attributes that this item has.  This
    * method will always return an array; if the item has no attributes
    * at all, getAttributes() will return an empty array: [].
    *
    * @param {item} item The item to access attributes on.
    * @type Array
	*/      
    getAttributes: function(/* Object */ item) {
    },
    
    /** 
    * summary:
    * Returns true if the given *item* has a value for the given *attribute*.
    *
    * @param {Object} item The item to access attributes on.
    * @param {String} attribute The attribute to access represented as a string.
    * @type Boolean
    */
    hasAttribute: function(/* Object */ item, /* String */ attribute) {
    },
    
    /**
    * summary:
    * Returns true if *something* is an item and came from the store instance.  
    * Returns false if *something* is a literal, an item from another store instance, 
    * or is any object other than an item.
    *
    * @param {Object} anObject the object to check against
    * @returns {Boolean}
    */
    isItem: function(/* Object */ anObject) {
    },
    
    /**
    * summary:<p/>
    * Given a query and set of defined options, such as a start and count of items to return,
    * this method executes the query and makes the results available as data items.
    * The format and expectations of stores is that they operate in a generally asynchronous 
    * manner, therefore callbacks are always used to return items located by the fetch parameters.
    *
    * object will always be returned and is returned immediately.<p/>
    * The basic request is nothing more than the keyword args passed to fetch and 
    * an additional function attached, abort().  The returned request object may then be used 
    * to cancel a fetch.  All data items returns are passed through the callbacks defined in the 
    * fetch parameters and are not present on the 'request' object.<p/><p/>
    *
    * This does not mean that custom data can not add methods and properties to the request object
    * returned, only that the API does not require it.  For more info about the Request API, 
    * see dojo.data.api.Request<p/><p/>
    *
	* The fetch() method will return a javascript object conforming to the API
    *        defined in dojo.data.api.Request.  In general, it will be the keywordArgs
    *        object returned with the required functions in Request.js attached.
    *        Its general purpose is to provide a convenient way for a caller to abort an
    *        ongoing fetch.  <p/><p/>
    * 
    *        The Request object may also have additional properties when it is returned
    *        such as request.store property, which is a pointer to the datastore object that 
    *        fetch() is a method of.  <p/><p/> 
    * 
    * @param {Object} keywordArgs
    * 	The keywordArgs parameter may either be an instance of 
    *   conforming to dojo.data.api.Request or may be a simple anonymous object
    *   that may contain any of the following: <p /><p/>
    *
    *            { 
    *                onBegin: Function,<p/>
    *                onItem: Function,<p/>
    *                onComplete: Function,<p/>
    *                onError: Function,<p/>
    *                scope: object,<p/>
    *                start: int<p/>
    *                count: int<p/>
    *                sort: array<p/>
    *            }<p/>
    *
    *   All implementations should accept keywordArgs objects with any of
    *   the 9 standard properties: query, onBegin, onItem, onComplete, onError 
    *   scope, sort, start, and count.  Some implementations may accept additional 
    *   properties in the keywordArgs object as valid parameters, such as 
    *   {includeOutliers:true}.<p/><p/>         
    *
    *   The *onBegin* parameter.<p/>
    *   	function(size, request);<p/>
    *   	If an onBegin callback function is provided, the callback function
    *   	will be called just once, before the first onItem callback is called.
    *       The onBegin callback function will be passed two arguments, the
    *       the total number of items identified and the Request object.  If the total number is
    *       unknown, then size will be -1.  Note that size is not necessarily the size of the 
    *       collection of items returned from the query, as the request may have specified to return only a 
    *       subset of the total set of items through the use of the start and count parameters.<p/><p/>
    *    
    *    The *onItem* parameter.<p/>
    *       function(item, request);<p/>
    *       If an onItem callback function is provided, the callback function
    *       will be called as each item in the result is received. The callback 
    *       function will be passed two arguments: the item itself, and the
    *       Request object.<p/><p/>
    *    
    *    The *onComplete* parameter.<p/>
    *       function(items, request);<p/><p/>
    *
    *        If an onComplete callback function is provided, the callback function
    *        will be called just once, after the last onItem callback is called.
    *        Note that if the onItem callback is not present, then onComplete will be passed
    *        an array containing all items which matched the query and the request object.  
    *        If the onItem callback is present, then onComplete is called as: 
    *        onComplete(null, request).<p/><p/>
    *
    *    The *onError* parameter.<p/>
    *        function(errorData, request);<p/> 
    *        If an onError callback function is provided, the callback function
    *        will be called if there is any sort of error while attempting to
    *        execute the query.
    *        The onError callback function will be passed two arguments:
    *        an Error object and the Request object.<p/><p/>
    *
    *    The *scope* parameter.<p/>
    *        If a scope object is provided, all of the callback functions (onItem, 
    *        onComplete, onError, etc) will be invoked in the context of the scope
    *        object.  In the body of the callback function, the value of the "this"
    *        keyword will be the scope object.   If no scope object is provided,
    *        the callback functions will be called in the context of dojo.global().  
    *        For example, onItem.call(scope, item, request) vs. 
    *        onItem.call(dojo.global(), item, request)<p/><p/>
    *
    *    The *start* parameter. <p/>
    *        If a start parameter is specified, this is a indication to the datastore to 
    *        only start returning items once the start number of items have been located and
    *        skipped.  When this parameter is paired withh 'count', the store should be able
    *        to page across queries with millions of hits by only returning subsets of the 
    *        hits for each query<p/><p/>
    *
    *    The *count* parameter.<p/>
    *        If a count parameter is specified, this is a indication to the datastore to 
    *        only return up to that many items.  This allows a fetch call that may have 
    *        millions of item matches to be paired down to something reasonable.<p/><p/>  
    *
    *    The *sort* parameter.<p/>
    *        If a sort parameter is specified, this is a indication to the datastore to 
    *        sort the items in some manner before returning the items.  The array is an array of 
    *        javascript objects that must conform to the following format to be applied to the
    *        fetching of items:<p/><p/>
    *        {
    *            attribute: attribute || attribute-name-string,
    *            descending: true|false;   // Optional.  Default is false.
    *        }
    *        Note that when comparing attributes, if an item contains no value for the attribute
    *        (undefined), then it the default ascending sort logic should push it to the bottom 
    *        of the list.  In the descending order case, it such items should appear at the top of the list.
    * @type Request
    */
    fetch: function(/* Object */ keywordArgs) {
    },
    
    /**
    * This method will serialize the data object to a format that can be persisted.
    * @returns {Object} a serialized data object. For most of the data this should return a human readable string.
    */
    serialize: function () {
    }    
});
