-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
--
-- Run the script in the sqlcmd command line processor
--
-- USAGE:  sqlcmd -U <userid> -P <password> -e -i preMigrateBusinessSpace700.sql
--
-- ------------------------------------------------------------------------------------
USE @DB_NAME@;
GO
CREATE SCHEMA BSPMGTMP AUTHORIZATION dbo;
GO

-- *****************************************************************
--
-- SCHEMA VERSION (100)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.VERSION_INFO(
    OID             NVARCHAR(64) NOT NULL,
    VERSION_MAJOR   INT,
    VERSION_MINOR   INT,
    VERSION_RELEASE INT,
    VERSION_DEV     INT,
    VERSION_TIME    BIGINT,
    DESCRIPTION     NVARCHAR(254),
    CONSTRAINT PK100 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- ACCOUNT (200)
--
-- *****************************************************************
-- 04/04 Accounts were ordered dropped.
--  The expectation is a userId is provided to persistence code
--  which is a vmm char(64) unique id. Susbsystems will create
--  required objects on the fly. Maybe want more centralized
--  provisioning to faciliate admin?
--CREATE TABLE BSPMGTMP.ACCOUNT (
--    OID         NVARCHAR(64)  NOT NULL,
--    EXTERNAL_ID NVARCHAR(255) NOT NULL,
--    NAME        NVARCHAR(255) NOT NULL,
--    CREATED     BIGINT        NOT NULL,
--    MODIFIED    BIGINT        NOT NULL,
--    STATE       INT DEFAULT 1 NOT NULL,
--    CONSTRAINT PK200 PRIMARY KEY (OID)
--);

-- *****************************************************************
--
-- CATALOG (300)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG (
    OID         NVARCHAR(64)  NOT NULL,
    ACCOUNT_OID NVARCHAR(1024) NOT NULL,
    UNIQUE_NAME NVARCHAR(255) NOT NULL,
    CREATED     BIGINT        NOT NULL,
    MODIFIED    BIGINT        NOT NULL,
    STATE       INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK300 PRIMARY KEY (OID)
);

CREATE TABLE BSPMGTMP.CATALOG_LOD (
    RESOURCE_OID NVARCHAR(64)   NOT NULL,
    LOCALE       NVARCHAR(64)   NOT NULL,
    TITLE        NVARCHAR(255)  NOT NULL,
    DESCRIPTION  NVARCHAR(1024) ,
    CONSTRAINT PK301 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK302 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.CATALOG (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- CATALOG ENTRY (310)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_ENTRY (
    OID            			NVARCHAR(64)   NOT NULL,
    CATALOG_OID    			NVARCHAR(64)   NOT NULL,
    CATEGORY_OID   			NVARCHAR(64)   NOT NULL,
    DEFINITION_URL 			NVARCHAR(2083) NOT NULL,
    CONTENT_URL    			NVARCHAR(2083) ,
    PREVIEW_URL    			NVARCHAR(2083) ,
    PREVIEW_THUMBNAIL_URL	NVARCHAR(2083)      ,
    HELP_URL       			NVARCHAR(2083)      ,    
    ICON_URL       			NVARCHAR(2083) ,
    HUB_ENTRY_URL  			NVARCHAR(2083) ,
    DEF_URL_TYPE   			INT    ,
    ICON_URL_TYPE  			INT    ,
    CREATED        			BIGINT NOT NULL,
    MODIFIED       			BIGINT NOT NULL,
    STATE          			INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK310 PRIMARY KEY (OID)
);

CREATE TABLE BSPMGTMP.CATALOG_ENTRY_LOD (
    RESOURCE_OID  		NVARCHAR(64)   NOT NULL,
    LOCALE        		NVARCHAR(64)   NOT NULL,
    TITLE         		NVARCHAR(255)  NOT NULL,
    DESCRIPTION   		NVARCHAR(1024) ,
    SHORT_DESCRIPTION   NVARCHAR(255)  ,
    CONSTRAINT PK311 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK311 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.CATALOG_ENTRY (OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.CATALOG_ENTRY_DD (
    RESOURCE_OID  NVARCHAR(64)   NOT NULL,
    NAME          NVARCHAR(255)  NOT NULL,
    VALUE         NVARCHAR(3000) ,
    CONSTRAINT PK312 PRIMARY KEY (RESOURCE_OID, NAME),
    CONSTRAINT FK312 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.CATALOG_ENTRY (OID) ON DELETE CASCADE
);


-- *****************************************************************
--
-- CATALOG CATEGORY (320)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_CATEGORY (
    OID         NVARCHAR(64)  NOT NULL,
    CATALOG_OID NVARCHAR(64)  NOT NULL,
    NAME        NVARCHAR(255) ,
    CREATED     BIGINT        NOT NULL,
    MODIFIED    BIGINT        NOT NULL,
    STATE       INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK320 PRIMARY KEY (OID),
    CONSTRAINT FK320 FOREIGN KEY (CATALOG_OID) REFERENCES BSPMGTMP.CATALOG (OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.CATALOG_CATEGORY_LOD (
    RESOURCE_OID NVARCHAR(64)   NOT NULL,
    LOCALE       NVARCHAR(64)   NOT NULL,
    TITLE        NVARCHAR(255)  NOT NULL,
    DESCRIPTION  NVARCHAR(1024) ,
    CONSTRAINT PK321 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK321 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.CATALOG_CATEGORY (OID) ON DELETE CASCADE
);


-- *****************************************************************
--
-- CATALOG CATEGORY INCLUDES (330)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_CATEGORY_INC (
    OID          		NVARCHAR(64)   NOT NULL,
    CATALOG_OID  		NVARCHAR(64)   NOT NULL,
    CATEGORY_OID 		NVARCHAR(64)   NOT NULL,
    INCLUDE_CAT  		NVARCHAR(255)  NOT NULL,
    INCLUDE_CATEGORY	NVARCHAR(255)  NOT NULL,
    CONSTRAINT PK330 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- CATALOG INCLUDES (340)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_INC (
    OID          		NVARCHAR(64)   NOT NULL,
    CATALOG_OID  		NVARCHAR(64)   NOT NULL,
    INCLUDE_CAT  		NVARCHAR(255)  NOT NULL,
    CONSTRAINT PK340 PRIMARY KEY (OID)
);
-- *****************************************************************
--
-- PREFERENCE (400)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.PREFERENCE (
    OID          NVARCHAR(64)  NOT NULL,
    UNIQUE_NAME  NVARCHAR(255) NOT NULL,
    PARENT_OID   NVARCHAR(64)  NOT NULL,
    TYPE         INT           NOT NULL,
    WDGT_INST_ID NVARCHAR(255) NOT NULL,
    ACCOUNT_ID   NVARCHAR(1024) NOT NULL,
    PAGE_ID      NVARCHAR(255) NOT NULL,
    CREATED      BIGINT        NOT NULL,
    MODIFIED     BIGINT        NOT NULL,
    STATE        INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK401 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- PREFERENCE NODE (WAS USED TO DEAL WITH COMPOSITE(NESTED)
--                  PREFERENCES, UNTIL DROPPED - HERE FOR REFERENCE)
--
-- *****************************************************************
--CREATE TABLE BSPMGTMP.PREFERENCE_NODE (
--    OID         NVARCHAR(64)  NOT NULL,
--    UNIQUE_NAME NVARCHAR(255) NOT NULL,
--    PREF_OID    NVARCHAR(64)  NOT NULL,
--    PARENT_OID  NVARCHAR(64)  NOT NULL,
--    TYPE        INT           NOT NULL,
--    CREATED     BIGINT        NOT NULL,
--    MODIFIED    BIGINT        NOT NULL,
--    STATE       INT DEFAULT 1 NOT NULL,
--    CONSTRAINT PK111 PRIMARY KEY (OID),
--    CONSTRAINT FK111 FOREIGN KEY (PREF_OID) REFERENCES BSPMGTMP.PREFERENCE (OID) ON DELETE CASCADE
--);

-- *****************************************************************
--
-- PREFERENCE NODE VALUE (410)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.PREFERENCE_VALUE (
    OID           NVARCHAR(64)   NOT NULL,
    PREF_OID      NVARCHAR(64)   NOT NULL,
    NAME          NVARCHAR(255)  NOT NULL,
    READ_ONLY     INT DEFAULT 0  NOT NULL,
    IS_LARGE      INT DEFAULT 0  NOT NULL,
    HAS_LIST      INT DEFAULT 0  NOT NULL,
    VALUE         NVARCHAR(1024) ,
    LARGE_VALUE   NVARCHAR(MAX)  ,
    STATE         INT DEFAULT 1  NOT NULL,
    CONSTRAINT PK410 PRIMARY KEY (OID),
    CONSTRAINT FK410 FOREIGN KEY (PREF_OID) REFERENCES BSPMGTMP.PREFERENCE (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- NAVIGATION NODE (500)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.NAVNODE (
    OID            NVARCHAR(64)  NOT NULL,
    OWNER_OID      NVARCHAR(1024) NOT NULL,
    THEME_ID       NVARCHAR(255) NOT NULL,
    THEME_NAME     NVARCHAR(255) NOT NULL,
    CONTENT        VARBINARY(MAX),
    BYTE_COUNT     INT           ,
    IS_ZIPPED      INT DEFAULT 0 NOT NULL,
    COMMUNITY_OID  NVARCHAR(64)  ,
    SPACE_OID      NVARCHAR(64)  ,
    CREATED        BIGINT        NOT NULL,
    MODIFIED       BIGINT        NOT NULL,
    STATE          INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK500 PRIMARY KEY (OID, OWNER_OID)
);

CREATE TABLE BSPMGTMP.NAVNODE_LOD (
    RESOURCE_OID  NVARCHAR(64)   NOT NULL,
    OWNER_OID     NVARCHAR(1024) NOT NULL,
    LOCALE        NVARCHAR(64)   NOT NULL,
    TITLE         NVARCHAR(255)  NOT NULL,
    DESCRIPTION   NVARCHAR(1024) ,
    CONSTRAINT FK500 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES BSPMGTMP.NAVNODE (OID,OWNER_OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.NAVNODE_DD (
    RESOURCE_OID  NVARCHAR(64)    NOT NULL,
    OWNER_OID     NVARCHAR(1024)  NOT NULL,
    NAME          NVARCHAR(255)   NOT NULL,
    VALUE         NVARCHAR(3000)  NOT NULL,
    CONSTRAINT PK502 PRIMARY KEY (RESOURCE_OID,OWNER_OID,NAME),
    CONSTRAINT FK502 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES BSPMGTMP.NAVNODE (OID,OWNER_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- NAVIGATION TREE (510)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.NAV_TREE (
    OWNER_OID    NVARCHAR(1024) NOT NULL,
    PAGE_OID     NVARCHAR(64)   NOT NULL,
    PARENT_OID   NVARCHAR(64)   NOT NULL,
    ORDINAL      INT  DEFAULT 1 ,
    IS_VIRTUAL   INT  DEFAULT 0 ,
    CONSTRAINT PK510 PRIMARY KEY (OWNER_OID, PAGE_OID)
);

-- *****************************************************************
--
-- ACCESS CONTROL (600)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.COMMUNITY_DEF (
    OID            NVARCHAR(64)  NOT NULL,
    OWNER_UID      NVARCHAR(1024) NOT NULL,
    RESOURCE_OID   NVARCHAR(64)  NOT NULL,
    RESOURCE_TYPE  NVARCHAR(32)  NOT NULL,
    CREATED        BIGINT        NOT NULL,
    MODIFIED       BIGINT        NOT NULL,
    STATE          INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK600 PRIMARY KEY( OID )
);
-- UNIQUE CONSTRAINT
ALTER TABLE BSPMGTMP.COMMUNITY_DEF ADD CONSTRAINT UQ600 UNIQUE (OID,RESOURCE_OID);

CREATE TABLE BSPMGTMP.ACL (
    COMMUNITY_OID     NVARCHAR(64)  NOT NULL,
    PARTICIPANT_UID   NVARCHAR(1024) NOT NULL,
    PARTICIPANT_TYPE  INT           NOT NULL,
    PERMISSIONS       NVARCHAR(255) NOT NULL,
    CREATED           BIGINT        NOT NULL,
    MODIFIED          BIGINT        NOT NULL,
    STATE             INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK610 PRIMARY KEY (COMMUNITY_OID,PARTICIPANT_UID),
    CONSTRAINT FK611 FOREIGN KEY (COMMUNITY_OID) REFERENCES BSPMGTMP.COMMUNITY_DEF (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- SPACES (700)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.SPACENODE (
    OID            NVARCHAR(64)        NOT NULL,
    OWNER_OID      NVARCHAR(1024)      NOT NULL,
    ROOTNAV_OID    NVARCHAR(64)        ,
    TEMPLATE_OID   NVARCHAR(64)        ,
    CREATED        BIGINT        NOT NULL,
    MODIFIED       BIGINT        NOT NULL,
    STATE          INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK700 PRIMARY KEY (OID, OWNER_OID)
);

CREATE TABLE BSPMGTMP.SPACENODE_LOD (
    RESOURCE_OID  NVARCHAR(64)    NOT NULL,
    OWNER_OID     NVARCHAR(1024)  NOT NULL,
    LOCALE        NVARCHAR(64)    NOT NULL,
    TITLE         NVARCHAR(255)   NOT NULL,
    DESCRIPTION   NVARCHAR(1024)          ,
    CONSTRAINT PK701 PRIMARY KEY (RESOURCE_OID,OWNER_OID,LOCALE),
    CONSTRAINT FK701 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES BSPMGTMP.SPACENODE (OID,OWNER_OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.SPACENODE_DD (
    RESOURCE_OID  NVARCHAR(64)    NOT NULL,
    OWNER_OID     NVARCHAR(1024)  NOT NULL,
    NAME          NVARCHAR(255)   NOT NULL,
    VALUE         NVARCHAR(3000)  NOT NULL,
    CONSTRAINT PK702 PRIMARY KEY (RESOURCE_OID,OWNER_OID,NAME),
    CONSTRAINT FK702 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES BSPMGTMP.SPACENODE (OID,OWNER_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- TEMPLATES (800)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.TEMPLATENODE (
    OID            NVARCHAR(64)        NOT NULL,
    OWNER_OID      NVARCHAR(1024),
    CONTENT        VARBINARY(MAX),
    BYTE_COUNT     INT           ,
    TEMPLATE_TYPE  NVARCHAR(64)  ,
    CREATED        BIGINT        NOT NULL,
    MODIFIED       BIGINT        NOT NULL,
    STATE          INT DEFAULT 1 NOT NULL,
    CONSTRAINT PK800 PRIMARY KEY (OID)
);

CREATE TABLE BSPMGTMP.TEMPLATENODE_LOD (
    RESOURCE_OID  NVARCHAR(64)    NOT NULL,
    LOCALE        NVARCHAR(64)    NOT NULL,
    TITLE         NVARCHAR(255)   NOT NULL,
    DESCRIPTION   NVARCHAR(1024)          ,
    CONSTRAINT PK801 PRIMARY KEY (RESOURCE_OID,LOCALE),
    CONSTRAINT FK801 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.TEMPLATENODE (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- ACCESS CONTROL RESOURCES (900)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.AC_RESOURCE (
    OID            NVARCHAR(64)       NOT NULL,
    OWNER_UID      NVARCHAR(1024)     NOT NULL,
    RESOURCE_OID   NVARCHAR(64)       NOT NULL,
    RESOURCE_TYPE  NVARCHAR(32)       NOT NULL,
    ROLE_BLOCKS    NVARCHAR(1024)             ,
    CREATED        BIGINT             NOT NULL,
    MODIFIED       BIGINT             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK900 PRIMARY KEY( RESOURCE_OID )
);

CREATE TABLE BSPMGTMP.AC_ROLE (
    AC_RESOURCE_OID   NVARCHAR(64)       NOT NULL,
    ROLE_OID          NVARCHAR(64)       NOT NULL,
    ROLE_NAME         NVARCHAR(1024)     NOT NULL,
    ROLE_TYPE         NVARCHAR(1024)             ,
    CREATED           BIGINT             NOT NULL,
    MODIFIED          BIGINT             NOT NULL,
    STATE             INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK910 PRIMARY KEY( ROLE_OID ),
    CONSTRAINT FK911 FOREIGN KEY (AC_RESOURCE_OID) REFERENCES BSPMGTMP.AC_RESOURCE (RESOURCE_OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.AC_MEMBER (
    ROLE_OID          NVARCHAR(64)       NOT NULL,
    MEMBER_UID        NVARCHAR(1024)     NOT NULL,
    MEMBER_TYPE       INT                NOT NULL,
    CONSTRAINT PK920 PRIMARY KEY (ROLE_OID,MEMBER_UID),
    CONSTRAINT FK921 FOREIGN KEY (ROLE_OID) REFERENCES BSPMGTMP.AC_ROLE (ROLE_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- FILESTORE (1000)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.FILESTORE_PATH (
    OID         NVARCHAR(64)           NOT NULL,
    PARENT_OID  NVARCHAR(64)           NOT NULL,
    NAME        NVARCHAR(255)          NOT NULL,
    PATH        NVARCHAR(255)          NOT NULL,
    LOCKED      NVARCHAR(64)           , 
    OWNER       NVARCHAR(64)           ,             
    CREATED     BIGINT                NOT NULL,
    MODIFIED    BIGINT                NOT NULL,
    COLL_TYPE   INT DEFAULT 1     NOT NULL,
    CONSTRAINT PK1010 PRIMARY KEY (OID)
);

CREATE TABLE BSPMGTMP.FILESTORE_FILE (
    OID         NVARCHAR(64)           NOT NULL,           
    F_SIZE      INT               NOT NULL,
    CONTENT     VARBINARY(MAX)
    CONSTRAINT PK1000 PRIMARY KEY (OID),
    CONSTRAINT FK1001 FOREIGN KEY (OID) REFERENCES BSPMGTMP.FILESTORE_PATH (OID) ON DELETE CASCADE
);


--*****************************************************************
--
-- Indexing
--
--*****************************************************************
-- Catalog
CREATE INDEX CATACCT      ON BSPMGTMP.CATALOG(ACCOUNT_OID);
CREATE INDEX ENTRYCATGOID ON BSPMGTMP.CATALOG_ENTRY(CATALOG_OID, CATEGORY_OID);
CREATE INDEX CATGCATOID   ON BSPMGTMP.CATALOG_CATEGORY(CATALOG_OID);
CREATE INDEX CATINTCATOID ON BSPMGTMP.CATALOG_INC(CATALOG_OID);

-- Preferences
CREATE INDEX PREFPARENT ON BSPMGTMP.PREFERENCE(PARENT_OID,ACCOUNT_ID);
CREATE INDEX PREFTYPE   ON BSPMGTMP.PREFERENCE(TYPE);

-- Navigation
CREATE INDEX PAGECOMM     ON BSPMGTMP.NAVNODE(COMMUNITY_OID);
CREATE INDEX PAGELODOWNRS ON BSPMGTMP.NAVNODE_LOD(OWNER_OID,RESOURCE_OID);
CREATE INDEX TREEOWPAR    ON BSPMGTMP.NAV_TREE(OWNER_OID,PARENT_OID);

-- Acl
CREATE INDEX COMMOWNPAR ON BSPMGTMP.COMMUNITY_DEF(OWNER_UID);
CREATE INDEX PRTTYUID   ON BSPMGTMP.ACL(PARTICIPANT_TYPE,PARTICIPANT_UID);
CREATE INDEX PRTUID     ON BSPMGTMP.ACL(PARTICIPANT_UID);

-- AC
CREATE INDEX AC_ROLE_RESOURCE ON BSPMGTMP.AC_ROLE(AC_RESOURCE_OID);
CREATE INDEX AC_MEMBER_UID ON BSPMGTMP.AC_MEMBER(MEMBER_UID);
CREATE INDEX AC_RESOURCE_OWNER ON BSPMGTMP.AC_RESOURCE(OWNER_UID,RESOURCE_OID);
